-------------------------------------------------------------------------------
-- Title      : VGA Syncgenerator
-- Project    : 
-------------------------------------------------------------------------------
-- File       : vga_sync.vhd
-- Author     : Konstantin
-- Company    : 
-- Created    : 2007-10-15
-- Last update: 2009-05-29
-- Platform   : 
-- Standard   : VHDL'93
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Copyright (c) 2007 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author          Description
-- 2007-10-15  1.0      Kest            Created
-- 2008-05-11  1.0      Kest            Timings hinzugefgt
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;


entity vga_sync is
  port(clk     : in  std_logic;
       hs      : out std_logic;
       vs      : out std_logic;
       pixel_x : out unsigned(9 downto 0);
       pixel_y : out unsigned(9 downto 0)
       );

end vga_sync;

architecture behave of vga_sync is
  

  constant cVGA_MODE : integer := 0;

  type tTimings is record
    a : time;
    b : time;
    c : time;
    d : time;

    a_vs : integer;
    b_vs : integer;
    c_vs : integer;
    d_vs : integer;


    clock : time;
  end record;

  type tvga_mode is array (0 to 1) of tTimings;

  constant vga_timings : tvga_mode := (
    (3.8 us, 1.9 us, 25.4 us, 0.6 us, 2, 33, 480, 10, 40 ns),
    (3.8 us, 1.9 us, 25.4 us, 0.6 us, 3, 25, 480, 1, 40 ns)

    );


  type tvga_timings_in_clocks is record
    a : integer;
    b : integer;
    c : integer;
    d : integer;

    a_vs : integer;
    b_vs : integer;
    c_vs : integer;
    d_Vs : integer;

    
  end record;

  constant vga : tvga_timings_in_clocks := (
    vga_timings(cVGA_MODE).a / vga_timings(cVGA_MODE).clock,
    vga_timings(cVGA_MODE).b / vga_timings(cVGA_MODE).clock,
    vga_timings(cVGA_MODE).c / vga_timings(cVGA_MODE).clock,
    vga_timings(cVGA_MODE).d / vga_timings(cVGA_MODE).clock,

    vga_timings(cVGA_MODE).a_vs,
    vga_timings(cVGA_MODE).b_vs,
    vga_timings(cVGA_MODE).c_vs,
    vga_timings(cVGA_MODE).d_vs

    );

  signal h_count, v_count : unsigned(9 downto 0);



  
begin



  process(clk)
  begin
    if rising_edge(clk) then
      
      if h_count < vga.a + vga.b + vga.c + vga.d then
        
        h_count <= h_count + 1;
        
      else
        
        h_count <= (others => '0');

        if v_count < vga.a_vs + vga.b_vs + vga.c_vs + vga.d_vs then
          v_count <= v_count + 1;
        else
          v_count <= (others => '0');
        end if;
        
      end if;


      if (h_count > vga.c + vga.d) and (h_count < vga.c + vga.d + vga.a) then
        hs <= '0';
      else
        hs <= '1';
      end if;


      if (v_count > vga.c_vs + vga.d_vs) and (v_count < vga.c_vs + vga.d_vs + vga.a_vs) then
        vs <= '0';
      else
        vs <= '1';
      end if;
      
    end if;
  end process;

  pixel_x <= h_count;
  pixel_y <= v_count;
  
end behave;
