-------------------------------------------------------------------------------
-- Title      : VGA Core
-- Project    : 
-------------------------------------------------------------------------------
-- File       : vgacore.vhd
-- Author     : 
-- Company    : 
-- Created    : 2007-10-16
-- Last update: 2009-05-29
-- Platform   : 
-- Standard   : VHDL'87
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Copyright (c) 2007 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2007-10-16  1.0      Kest    Created
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.numeric_std.all;

entity vgacore is
  port (
    nReset : in std_logic;
    clk_24 : in std_logic;

    VGA_Sync_o : out std_logic;
    VGA_DE : out std_logic;
    VGA_HS : out std_logic;
    VGA_VS : out std_logic;
    VGA_R  : out std_logic_vector(3 downto 0);
    VGA_G  : out std_logic_vector(3 downto 0);
    VGA_B  : out std_logic_vector(3 downto 0)

    );
end vgacore;

architecture behave of vgacore is

  component vga_sync
    port (
      clk     : in  std_logic;
      hs      : out std_logic;
      vs      : out std_logic;
      pixel_x : out unsigned(9 downto 0);
      pixel_y : out unsigned(9 downto 0));
  end component;

  component font8x8
    port (
      nReset    : in  std_logic;
      clk       : in  std_logic;
      char      : in  std_logic_vector(7 downto 0);
      char_line : in  std_logic_vector(2 downto 0);
      data_out  : out std_logic_vector(7 downto 0));
  end component;

  component char_page
    port (
      nReset   : in  std_logic;
      clk      : in  std_logic;
      wr_en    : in  std_logic;
      addr_wr  : in  std_logic_vector(12 downto 0);
      char_in  :     std_logic_vector(7 downto 0);
      addr_rd  : in  std_logic_vector(12 downto 0);
      char_out : out std_logic_vector(7 downto 0));
  end component;


  signal clk : std_logic;

  signal horiz_sync_out   : std_logic;
  signal vert_sync_out    : std_logic;
  signal pixel_x, pixel_y : unsigned(9 downto 0);

  -- Char

  signal char      : std_logic_vector(7 downto 0);
  signal char_line : std_logic_vector(2 downto 0);
  signal data_out  : std_logic_vector(7 downto 0);

  -- Char to VGA
  signal char_x         : integer range 0 to 7;
  signal vga_last_pixel : std_logic;
  signal ascii_code     : integer range 0 to 255;
  -- Page

  signal page_addr_rd  : std_logic_vector(12 downto 0);
  signal page_addr_wr  : std_logic_vector(12 downto 0);
  signal char_out      : std_logic_vector(7 downto 0);
  signal page_char_in  : std_logic_vector(7 downto 0);
  signal page_wr_en    : std_logic;
  signal char_to_write : unsigned(7 downto 0);
  
begin

  VGA_Sync_o <= vga_last_pixel;
  
  -----------------------------------------------------------------------------
  -- VGA Sync Signal Generator
  -----------------------------------------------------------------------------
  VGA_SYNC_1 : VGA_SYNC
    port map (
      clk     => clk_24,
      hs      => VGA_HS,
      vs      => VGA_VS,
      pixel_x => pixel_x,
      pixel_y => pixel_y);


  -----------------------------------------------------------------------------
  -- Chars on VGA
  -----------------------------------------------------------------------------


  
  char_page_1 : char_page
    port map (
      nReset   => nReset,
      clk      => clk_24,
      wr_en    => page_wr_en,
      addr_wr  => page_addr_wr,
      char_in  => page_char_in,
      addr_rd  => page_addr_rd,
      char_out => char_out);

  char <= char_out;

  process(clk_24)
  begin
    if rising_edge(clk_24) then
      
      page_addr_rd <= std_logic_vector(to_unsigned(to_integer((80 * ((pixel_y)/8)) + (pixel_x+10)/8)-1, 13));
      char_line    <= std_logic_vector(to_unsigned(to_integer((pixel_y mod 8)), 3));
      char_x       <= to_integer((pixel_x mod 8));
      
    end if;
  end process;


  -----------------------------------------------------------------------------
  -- Test Write To Page
  -----------------------------------------------------------------------------
  write_to_page : if 0 = 1 generate


    page_char_in <= std_logic_vector(char_to_write);

    process(clk_24)
    begin
      if rising_edge(clk_24) then
        if vga_last_pixel = '1' then
          page_wr_en    <= '1';
          char_to_write <= char_to_write + 1;

          if unsigned(page_addr_wr) < 4800 then
            page_addr_wr <= std_logic_vector(unsigned(page_addr_wr) + 1);
          else
            page_addr_wr <= (others => '0');
          end if;
          
        end if;
      end if;
    end process;
    
  end generate write_to_page;
  -----------------------------------------------------------------------------
  -- End of Test Write To Page
  -----------------------------------------------------------------------------


  process(clk_24)
  begin
    if rising_edge(clk_24) then

      -- FSycn
      if pixel_x = 639 and pixel_y = 479 then
        vga_last_pixel <= '1';
      else
        vga_last_pixel <= '0';
      end if;


      

      if (pixel_x >= 0 and pixel_x < 640 and
          pixel_y >= 0 and pixel_y < 480) then

        VGA_DE <= '1';
        

        if (pixel_x = 0 or pixel_x = 639 or
            pixel_y = 0 or pixel_y = 479 or
            pixel_x = 100


            ) then

          VGA_R <= (others => '1');
          VGA_G <= (others => '1');
          VGA_B <= (others => '1');
          
        else
          
          

          if data_out(7-char_x) = '1' then
            VGA_R <= (others => '0');
            VGA_G <= (others => '1');
            VGA_B <= (others => '0');
          else
            VGA_R <= (others => '0');
            VGA_G <= (others => '0');
            VGA_B <= (others => '0');
          end if;
          
        end if;

        
      else
        VGA_DE <= '0';
        
        VGA_R <= (others => '0');
        VGA_G <= (others => '0');
        VGA_B <= (others => '0');
      end if;

      
    end if;

  end process;



  font8x8_1 : font8x8
    port map (
      nReset    => nReset,
      clk       => clk_24,
      char      => char,
      char_line => char_line,
      data_out  => data_out);

end behave;
